// Object Image XDisplay
var XDisplay_Img = { 

    name: '',
    /* Coordonnes, angle de rotation et dimension d'affichage */
    x: 0, 
    y: 0,
    r: 0,
    width: 0,
    height: 0,
    hidden: false,
    
    /* Pour l'animation */
    tx_prev: 0,
    tx_anim: 0, //x et y pour animation avec images multiples
    ty_anim: 0,
    anim_current_frame: 0,
    anim_nb_frames: 0,
    time_anim: false, //timing de l'animation avec images multiples
    repeat_anim: true,
    callback_anim: null,
    
    /* Hitbox */
    hitbox_type: 'rect',
    width_hit: 0,
    height_hit: 0,
    
    img: null,
    
    animations: new Array(), 
    
    setName: function(name) {
      this.name = name;
    },
    
    setCoords: function(x, y) {
      this.x = x;
      this.y = y;
    },
    
    setRotation: function(r) {
      this.r = r;
    },
    
    setDimension: function(width, height) {
      this.width = width;
      this.height = height;
    },
    
    setHitbox: function(hitbox_type, width, height) {
      this.hitbox_type = hitbox_type;
      this.width_hit = width;
      this.height_hit = height;
    },
    
    setImg: function(img) {
      this.img = img;
    },
    
    setAnimation: function(ty_anim, nb_frames, time_anim, repeat_anim, callback) {
      if(this.ty_anim != ty_anim)
        this.anim_current_frame = 0;
      this.ty_anim = ty_anim;
      this.anim_nb_frames = nb_frames;
      this.time_anim = time_anim;
      if(repeat_anim != undefined)
        this.repeat_anim = repeat_anim;
      else
        this.repeat_anim = true;
      if(callback != undefined)
        this.callback_anim = callback;
      else
        this.callback_anim = null;
    },
    
    updateAnimation: function(time) {

      //delay entre chaque frame de l'animation
      var delay = this.time_anim/1000/this.anim_nb_frames;
      if(this.tx_anim+time-this.tx_prev > delay)
      {
        this.anim_current_frame += 1;
        this.tx_prev = time;
        this.tx_anim = 0;
        if(this.anim_current_frame >= this.anim_nb_frames)
        {
          //si l'animation est repte on remet  0
          if(this.repeat_anim)
            this.anim_current_frame = 0;
          //sinon si function de callback on l'appel
          else if(this.callback_anim != null)
          {
             this.callback_anim(this);
          }
          //sinon on affiche la dernire image et on dsactive l'animation
          else
          {
            this.time_anim = 0;
            this.anim_current_frame = this.anim_nb_frames-1;
          }
        }
      }
      if(time == 1)
        this.tx_anim += time;
        
    },
    
    clearAnimation: function(){
      this.time_anim = 0; 
    },
    
    draw: function(context, time) {
      if(this.hidden == false)
      {
        context.save();
        context.translate(this.x+this.width/2, this.y+this.height/2);
        context.rotate(this.r);
        context.translate(-this.width/2, -this.height/2);
        context.drawImage(this.img, this.anim_current_frame*this.width,  this.ty_anim*this.height, this.width, this.height, 0, 0, this.width, this.height);
        context.restore();
       } 
       /* Animation de l'objet si dfinit */
       if(this.time_anim > 0)
         this.updateAnimation(time);     
    }
}

// Object Geometric XDisplay
var XDisplay_Geo = { 

    name: '',
    type: 'rect', //rect, circle, ellipse
    /* Coordonnes, angle de rotation et dimension d'affichage */
    x: 0, 
    y: 0,
    r: 0,
    width: 0,
    height: 0,
    hidden: false,
    
    filled: false,
    color: 'rgb(0,0,0)',
    strokeWidth: 2,
    
    /* Hitbox */
    width_hit: 0,
    height_hit: 0,
    hitbox_type: 'rect',
    
    setName: function(name) {
      this.name = name;
    },
    
    setCoords: function(x, y) {
      this.x = x;
      this.y = y;
    },
    
    setRotation: function(r) {
      this.r = r;
    },
    
    setColor: function(color) {
      this.color = color;
    },
    
    setStrokeWidth: function(width) {
      this.strokeWidth = width;
    },
    
    setHitbox: function(hitbox_type, width, height) {
      this.hitbox_type = hitbox_type;
      this.width_hit = width;
      this.height_hit = height;
    },
    
    setRect: function(width, height, filled) {
      this.type = 'rect';
      this.width = width;
      this.height = height;
      this.filled = filled;
    },
    
    setCircle: function(width, filled) {
      this.type = 'circle';
      this.width = width;
      this.filled = filled;
    },
    
    setEllipse: function(width, height, filled) {
      this.type = 'ellipse';
      this.width = width;
      this.height = height;
      this.filled = filled;
    },
    
    
    draw: function(context, time) {
      if(this.hidden == false)
      {
        context.save();
        
        if(this.type == 'circle')
        {
          context.translate(this.x+this.width/2, this.y+this.width/2);
          context.beginPath();
          context.arc(0, 0, this.width/2, 0, Math.PI*2);
          if(this.filled)
          {
            context.fillStyle = this.color;
            context.fill();
          }
          else
          {
            context.strokeStyle = this.color;
            context.lineWidth = this.strokeWidth;
            context.stroke();
          }
        }
        else if(this.type == 'rect')
        {
          context.translate(this.x, this.y);
          context.rotate(this.r);
          context.beginPath();
          if(this.filled)
          {
            context.fillStyle = this.color;
            context.fillRect(0, 0, this.width, this.height);
          }
          else
          {
            context.strokeStyle = this.color;
            context.lineWidth = this.strokeWidth;
            context.strokeRect(0, 0, this.width, this.height);
          }
        }
        else if(this.type == 'ellipse')
        {
          var dy_ctrl = this.height/6; //delta y des points de control
          context.rotate(this.r);
          context.beginPath();
          context.moveTo(this.x, this.y+this.height/2);
          context.bezierCurveTo(this.x, this.y-dy_ctrl, this.x+this.width, this.y-dy_ctrl, this.x+this.width, this.y+this.height/2);
          context.bezierCurveTo(this.x+this.width, this.y+this.height+dy_ctrl, this.x, this.y+this.height+dy_ctrl, this.x, this.y+this.height/2);
          if(this.filled)
          {
            context.fillStyle = this.color;
            context.fill();
          }
          else
          {
            context.strokeStyle = this.color;
            context.lineWidth = this.strokeWidth;
            context.stroke();
          }
          context.closePath();
        }
        context.restore();
        /* Animation de l'objet si dfinit */
        if(this.time_anim > 0)
          this.updateAnimation(time);
      }     
    }
}

// Object Geometric XDisplay
var XDisplay_Text = {
    
    name: '',
    /* Coordonnes, angle de rotation et police d'affichage */
    x: 0, 
    y: 0,
    r: 0,
    hidden: false,
    fontSize: 12,
    fontFamily: 'Arial',
    color: 'rgb(0,0,0)',
     
    setName: function(name) {
      this.name = name;
    },
    
    setCoords: function(x, y) {
      this.x = x;
      this.y = y;
    },
    
    setRotation: function(r) {
      this.r = r;
    },
    
    setColor: function(color) {
      this.color = color;
    },
    
    setFontSize: function(size) {
      this.fontSize = size;
    },
    
    setFontFamily: function(family) {
      this.fontFamily = family;
    },
    
    setText: function(text) {
      this.text = text;
    },
    
    draw: function(context, time) {
      if(this.hidden == false)
      {
        context.save();
        context.fillStyle = this.color;
        context.font = this.fontSize+'pt '+this.fontFamily;
        var texts = this.text.split("\r\n");
        var s = texts.length;
        for(var i=0; i<s; i++)
          context.fillText(texts[i], this.x, this.y+i*(this.fontSize+this.fontSize/2));
      }
    }
}